﻿//
// Mecanimのアニメーションデータが、原点で移動しない場合の Rigidbody付きコントローラ
// サンプル
// 2014/03/13 N.Kobyasahi
//
using UnityEngine;
using System.Collections;

// 必要なコンポーネントの列記
[RequireComponent(typeof (Animator))]
[RequireComponent(typeof (CapsuleCollider))]
[RequireComponent(typeof (Rigidbody))]

public class UnityChanControlScriptWithRgidBody : MonoBehaviour
{

	public float animSpeed = 1.5f;				// アニメーション再生速度設定
	public float lookSmoother = 3.0f;			// a smoothing setting for camera motion
	public bool useCurves = true;				// Mecanimでカーブ調整を使うか設定する
												// このスイッチが入っていないとカーブは使われない
	public float useCurvesHeight = 0.5f;		// カーブ補正の有効高さ（地面をすり抜けやすい時には大きくする）

	// 以下キャラクターコントローラ用パラメタ
	// 前進速度
	public float forwardSpeed = 7.0f;
	// 旋回速度
	float rotateSpeed = 360.0f;

	


	// ジャンプ威力
	public float jumpPower = 3.0f; 
	// キャラクターコントローラ（カプセルコライダ）の参照
	private CapsuleCollider col;
	private Rigidbody rb;
	// キャラクターコントローラ（カプセルコライダ）の移動量
	//private Vector3 velocity;
	// CapsuleColliderで設定されているコライダのHeiht、Centerの初期値を収める変数
	private float orgColHight;
	private Vector3 orgVectColCenter;
	
	private Animator anim;							// キャラにアタッチされるアニメーターへの参照
	private AnimatorStateInfo currentBaseState;			// base layerで使われる、アニメーターの現在の状態の参照


	Camera cameraMain;
	Transform _transform;

	public float angleY=0.0f;

		
// アニメーター各ステートへの参照
	static int idleState = Animator.StringToHash("Base Layer.Idle");
	static int locoState = Animator.StringToHash("Base Layer.Locomotion");
	static int jumpState = Animator.StringToHash("Base Layer.Jump");
	static int restState = Animator.StringToHash("Base Layer.Rest");

// 初期化
	void Start ()
	{
		// Animatorコンポーネントを取得する
		anim = GetComponent<Animator>();
		// CapsuleColliderコンポーネントを取得する（カプセル型コリジョン）
		col = GetComponent<CapsuleCollider>();
		rb = GetComponent<Rigidbody>();
		
		//メインカメラを取得する
		cameraMain = Camera.main;
		_transform = this.transform;


		// CapsuleColliderコンポーネントのHeight、Centerの初期値を保存する
		orgColHight = col.height;
		orgVectColCenter = col.center;
		
	}
	
	
	// 以下、メイン処理.リジッドボディと絡めるので、FixedUpdate内で処理を行う.
	void FixedUpdate ()
	{
		float deadZone = 0.1f;

		float inputHorizontal = Input.GetAxis("Horizontal");	// 入力デバイスの水平軸をhで定義
		float inputVertical = Input.GetAxis("Vertical");		// 入力デバイスの垂直軸をvで定義


		//カメラ方向を黄色の線で表示。
		Vector3 cameraDirection = cameraMain.transform.rotation * new Vector3(0, 0, 2.0f);
		cameraDirection.y = 0.0f;
		Debug.DrawLine(_transform.position, _transform.position + cameraDirection, Color.yellow);

		//プレイヤーの方向を緑の線で表示。
		Vector3 playerDirection = _transform.rotation * new Vector3(0, 0, 4.0f);
		playerDirection.y = 0.0f;
		Debug.DrawLine(_transform.position, _transform.position + playerDirection, Color.green);


		//左スティックの移動量を赤線で表示。
		Vector3 movement = new Vector3(inputHorizontal, 0, inputVertical);
		Debug.DrawLine(_transform.position, _transform.position + movement * 4.0f, Color.red);

		//移動方向を青線で表示。
		movement = cameraMain.transform.rotation * movement;
		movement.y = 0.0f;

		anim.SetFloat("Speed", movement.magnitude);		// Animator側で設定している"Speed"パラメタにvを渡す
		//anim.SetFloat("Direction", inputHorizontal); 	// Animator側で設定している"Direction"パラメタにhを渡す
		anim.speed = animSpeed;							// Animatorのモーション再生速度に animSpeedを設定する
		currentBaseState = anim.GetCurrentAnimatorStateInfo(0);	// 参照用のステート変数にBase Layer (0)の現在のステートを設定する
		rb.useGravity = true;//ジャンプ中に重力を切るので、それ以外は重力の影響を受けるようにする


		/*
		if (Input.GetButtonDown("Jump"))
		{	// スペースキーを入力したら

			Debug.Log("Jump");

			//アニメーションのステートがLocomotionの最中のみジャンプできる
			if (currentBaseState.nameHash == locoState)
			{
				//ステート遷移中でなかったらジャンプできる
				if (!anim.IsInTransition(0))
				{
					rb.AddForce(Vector3.up * jumpPower, ForceMode.VelocityChange);
					anim.SetBool("Jump", true);		// Animatorにジャンプに切り替えるフラグを送る
				}
				else
					Debug.Log("Now transition...");
			}
			else
			{
				Debug.Log("Not " + locoState);
			}
		}
		*/


		if (movement.magnitude > deadZone)
		{
			Debug.DrawLine(_transform.position, _transform.position + movement * forwardSpeed, Color.cyan);
			
			angleY = GetSignedAngle(_transform.forward, movement, _transform.up);

			//移動
			_transform.position += movement * forwardSpeed * Time.fixedDeltaTime;

			//回転
			float rotY = Mathf.Lerp(angleY, 0, 0.8f);
			_transform.Rotate(0, rotY, 0);

		}
		else
		{
			Debug.DrawLine(_transform.position, _transform.position + movement * forwardSpeed, Color.magenta);
		}
		
		
		// 以下、Animatorの各ステート中での処理
		// Locomotion中
		// 現在のベースレイヤーがlocoStateの時
		if (currentBaseState.nameHash == locoState){
			//カーブでコライダ調整をしている時は、念のためにリセットする
			if(useCurves){
				resetCollider();
			}
		}
		// JUMP中の処理
		// 現在のベースレイヤーがjumpStateの時
		else if(currentBaseState.nameHash == jumpState)
		{
			//MEMO:カメラを変えているのでコメントアウト。
			//cameraObject.SendMessage("setCameraPositionJumpView");	// ジャンプ中のカメラに変更


			// ステートがトランジション中でない場合
			if(!anim.IsInTransition(0))
			{
				
				// 以下、カーブ調整をする場合の処理
				if(useCurves){
					// 以下JUMP00アニメーションについているカーブJumpHeightとGravityControl
					// JumpHeight:JUMP00でのジャンプの高さ（0〜1）
					// GravityControl:1⇒ジャンプ中（重力無効）、0⇒重力有効
					float jumpHeight = anim.GetFloat("JumpHeight");
					float gravityControl = anim.GetFloat("GravityControl"); 
					if(gravityControl > 0)
						rb.useGravity = false;	//ジャンプ中の重力の影響を切る
										
					// レイキャストをキャラクターのセンターから落とす
					Ray ray = new Ray(transform.position + Vector3.up, -Vector3.up);
					RaycastHit hitInfo = new RaycastHit();
					// 高さが useCurvesHeight 以上ある時のみ、コライダーの高さと中心をJUMP00アニメーションについているカーブで調整する
					if (Physics.Raycast(ray, out hitInfo))
					{
						if (hitInfo.distance > useCurvesHeight)
						{
							col.height = orgColHight - jumpHeight;			// 調整されたコライダーの高さ
							float adjCenterY = orgVectColCenter.y + jumpHeight;
							col.center = new Vector3(0, adjCenterY, 0);	// 調整されたコライダーのセンター
						}
						else{
							// 閾値よりも低い時には初期値に戻す（念のため）					
							resetCollider();
						}
					}
				}
				// Jump bool値をリセットする（ループしないようにする）				
				anim.SetBool("Jump", false);
			}
		}
		// IDLE中の処理
		// 現在のベースレイヤーがidleStateの時
		else if (currentBaseState.nameHash == idleState)
		{
			//カーブでコライダ調整をしている時は、念のためにリセットする
			if(useCurves){
				resetCollider();
			}
			// スペースキーを入力したらRest状態になる
			if (Input.GetButtonDown("Jump")) {
				anim.SetBool("Rest", true);
			}
		}
		// REST中の処理
		// 現在のベースレイヤーがrestStateの時
		else if (currentBaseState.nameHash == restState)
		{
			//cameraObject.SendMessage("setCameraPositionFrontView");		// カメラを正面に切り替える
			// ステートが遷移中でない場合、Rest bool値をリセットする（ループしないようにする）
			if(!anim.IsInTransition(0))
			{
				anim.SetBool("Rest", false);
			}
		}
	}




	void OnGUI()
	{
		int labelHeight = 30;

		GUI.Box(new Rect(Screen.width - 260, 10, 250, 90), "Interaction");
		GUI.Label(new Rect(Screen.width - 245, 30, 250, labelHeight), "left stick : move");
		GUI.Label(new Rect(Screen.width - 245, 50, 250, labelHeight), "right stick : camera");
		GUI.Label(new Rect(Screen.width - 245, 70, 250, labelHeight), "L/R button: camera zoom");
		//GUI.Label(new Rect(Screen.width - 245, 70, 250, labelHeight), "triangle button while Running : Jump");
		//GUI.Label(new Rect(Screen.width - 245, 70, 250, labelHeight), "triangle button while Stopping: Rest");
		
		
	}


	// キャラクターのコライダーサイズのリセット関数
	void resetCollider()
	{
	// コンポーネントのHeight、Centerの初期値を戻す
		col.height = orgColHight;
		col.center = orgVectColCenter;
	}



	/// <summary>fwdからみてtargetDirectionは左側か右側かを判定します。</summary>
	/// <param name="fwd"></param>
	/// <param name="targetDirection"></param>
	/// <param name="up"></param>
	/// <returns>1なら右、-1なら左、0なら中央。</returns>
	static public int TargetSide2(Vector3 fwd, Vector3 targetDirection, Vector3 up)
	{
		Vector3 perp = Vector3.Cross(fwd, targetDirection);
		float dir = Vector3.Dot(perp, up);

		if (dir > 0.0)
			return 1;
		else if (dir < 0.0)
			return -1;
		else
			return 0;
	}


	static public float GetSignedAngle(Vector3 fwd, Vector3 targetDirection, Vector3 up)
	{
		return Vector3.Angle(fwd, targetDirection) * TargetSide2(fwd, targetDirection, up);

	}


}
